package com.extensiblejava.core.employee.builder.imp;

import com.extensiblejava.core.employee.Employee;
import com.extensiblejava.core.employee.Name;
import com.extensiblejava.core.employee.builder.Builder;
import com.extensiblejava.core.employee.selectable.CompleteEmployeeSelectable;
import com.extensiblejava.dao.*;
import com.extensiblejava.dao.mediator.*;

import java.util.ArrayList;

public class CompleteBuilder implements Builder {
	private boolean isBuilt = false;
	private Integer empId;
	private Employee employee;
	private Employee manager;
	private Employee[] staff;
	public CompleteBuilder(Integer empId) {
		this.empId = empId;	
	}
	
	private Employee build(DataCursor cursor) throws DAOException {
		Integer empId = new Integer((String) cursor.getValue("ID"));
		Integer mgrId = new Integer((String) cursor.getValue("MGR_ID"));
		//this.startDate = (Date) cursor.getValue("START_DT");
		//this.endDate = (Date) cursor.getValue("END_DT");
		String dept = (String) cursor.getValue("DEPARTMENT_CD");
		String pos = (String) cursor.getValue("POSITION_CD");
		String type = (String) cursor.getValue("TYPE_CD");
		Double rate = new Double((String) cursor.getValue("RATE"));
		String firstName = (String) cursor.getValue("FIRST");
		String lastName = (String) cursor.getValue("LAST");
		Name name = new Name(firstName, lastName);
		return new Employee(empId, mgrId, dept, pos, type, rate, name, new CompleteBuilder(empId));
	}
	
	private void buildAll() {
		if (this.isBuilt) { return ;}
		this.isBuilt = true;
		try {
			DAOMediator mediator = new DAOMediator();
			CompleteEmployeeSelectable selectable = new CompleteEmployeeSelectable(empId);
			DataCursor cursor = mediator.select(selectable);
			
			ArrayList list = new ArrayList();
			while (cursor.next()) {
				String empType = (String) cursor.getValue("TYPE");
				if (empType.equals("EMP")) {
					this.employee = this.build(cursor);
				} else if (empType.equals("MGR")) {
					this.manager = this.build(cursor);
				} else if (empType.equals("STF")) {
					Employee e = build(cursor);
					list.add(e);
				}
			}
			this.staff = new Employee[list.size()];
			System.arraycopy(list.toArray(), 0, this.staff, 0, list.size());
			//this.staff = (Employee[]) list.toArray();
		} catch (TransactionException e) {
			e.printStackTrace();
		} catch (DAOException e) {
			e.printStackTrace();
		}	
	}
	public Employee buildEmployee() { 
		this.buildAll();
		return this.employee;
	}
	public Employee buildManager() { 
		this.buildAll();
		return this.manager;
	}
	public Employee[] buildStaff() {
		this.buildAll();
		return this.staff;
	}

}
