package com.extensiblejava.dao;
import java.sql.*;
import java.util.*;

public final class DataBaseDAO extends BaseDAO {
	private static DataBaseDAO instance;
	private DataBaseDAO() {}
	public synchronized static DataBaseDAO getInstance() {
		if (instance == null) {
			instance = new DataBaseDAO();	
		}
		return instance;		
	}
	
	public DataCursor executeSelect(Selectable selectable) throws DAOException {
		return this.executeSelect(selectable, null);
	}
	public DataCursor executeSelect(Selectable selectable, Transaction transaction)
		throws DAOException {

		PreparedStatement ps = null;
		Connection conn = null;

		try {
			if (transaction == null) {
				transaction = new Transaction();	
			}
			conn = transaction.getConnection();
			String sql = selectable.getSelectSQL();
			ps = conn.prepareStatement(sql);
			Iterator columnValues = selectable.getSortedSelectValues().values().iterator();
			int index = 1;
			while (columnValues.hasNext()) {
				Object value = columnValues.next();
				ps.setObject(index, value);
				index++;
			}

			ResultSet rs = ps.executeQuery();
			
			TabularRecordSet dc = new TabularRecordSet(rs);
			
			return dc;

		} catch (SQLException ex) {
			throw new DAOException(ex);
		} catch (Exception e) {
			throw new DAOException(e);	
		} finally {
			try {
				if (ps != null) { ps.close(); }
			} catch (SQLException e) {
				throw new DAOException("Error closing,", e);	
			}	
		}
	}
	
	public Integer executeInsert(Insertable persistable, Transaction trans)
			throws DAOException {

			//Variables declaration
			PreparedStatement ps = null;
			Integer key = null;

			try {
				Connection conn = trans.getConnection();
				String sql = persistable.getInsertSQL();
				ps = conn.prepareStatement(sql);
				Iterator columnValues = persistable.getSortedInsertValues().values().iterator();
				int index = 1;
				while (columnValues.hasNext()) {
					Object value = columnValues.next();
					ps.setObject(index, value);
					index++;
				}
				int retCode = ps.executeUpdate();

			} catch (SQLException ex) {
				throw new DAOException(ex);
			} catch (Exception e) {
				throw new DAOException(e);
			} finally {
				try {
					if (ps != null) { ps.close(); }
				} catch (SQLException e) {
					throw new DAOException("Error closing.", e);	
				}
			}
			return key;
		}
}