package com.extensiblejava.dao;

import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.ListIterator;

public class TabularRecordSet implements DataCursor {
	private int _numRows;
	private int _numColumns;
	private Object _currRow[];
	private ListIterator _rowTraverser;
	private ArrayList _rows;
	private Hashtable _columns = new Hashtable();

	public TabularRecordSet(Header header[]) {
		this._numColumns = header.length;
		for (int i = 0; i < header.length; i++) {
			this._columns.put(new Integer(i), header[i]);
		}
	}

	public TabularRecordSet(ResultSet resultSet) throws DAOException {
		try {

			ResultSetMetaData rsmd = resultSet.getMetaData();

			this._columns = new Hashtable();
			for (int i = 0; i < rsmd.getColumnCount(); i++) {
				String colName = rsmd.getColumnName(i + 1);
				int dataType = rsmd.getColumnType(i + 1);
				Header colHeader = new Header(colName, dataType, i + 1);
				//this._header[i] = colName;
				this._columns.put(new Integer(i), colHeader);
			}

			this._rows = new ArrayList();
			while (resultSet.next()) {
				int colCount = rsmd.getColumnCount();
				Object row[] = new Object[colCount];
				for (int currColumn = 0;
					currColumn < rsmd.getColumnCount();
					currColumn++) {
					int dataType = rsmd.getColumnType(currColumn + 1);
					Object colValue = resultSet.getObject(currColumn + 1);
					row[currColumn] = colValue;
				}
				this._rows.add(row);
			}
		} catch (SQLException e) {
			throw new DAOException(e);
		}
	}

	public void close() throws DAOException {
		this._currRow = null;
		this._columns = null;
		this._rowTraverser = null;
		this._rows = null;
	}
	public void createRow(Object row[]) {
		if (this._rows == null) {
			this._rows = new ArrayList();
		}
		this._rows.add(row);
	}
	public boolean first() throws DAOException {
		this._rowTraverser = this._rows.listIterator();
		this._currRow = (Object[]) this._rowTraverser.next();
		return true;
	}
	public String getColumnName(int colNum) throws DAOException {
		Header column = (Header) this._columns.get(new Integer(colNum - 1));
		return column.getColumnName();
	}
	public Object getMetaData() throws DAOException {
		return null;
	}
	public Object getResultSet() throws DAOException {
		return null;
	}
	public Object getValue(int colNum) throws DAOException {
		if (this._currRow == null) {
			return null;
		}
		return this._currRow[colNum - 1];
	}
	public Object getValue(String colName) throws DAOException {
		if (this._currRow == null) {
			return null;
		}

		int colNum = -1;
		for (int i = 0; i < this._columns.size(); i++) {
			Header header = (Header) this._columns.get(new Integer(i));
			if (header.getColumnName().equalsIgnoreCase(colName)) {
				colNum = i;
				break;
			}
		}

		if (colNum == -1) {
			throw new DAOException("Column " + colName + " not found.");
		}

		return this._currRow[colNum];
	}
	public boolean hasMetaData() {
		return false;
	}
	public boolean hasResultSet() {
		return false;
	}
	public boolean last() throws DAOException {
		this._rowTraverser = this._rows.listIterator(this._rows.size() - 1);
		this._currRow = (Object[]) this._rowTraverser.next();
		return true;
	}
	public boolean next() throws DAOException {
		if (this._rowTraverser == null) {
			this._rowTraverser = this._rows.listIterator();
		}

		if (this._rowTraverser.hasNext()) {
			this._currRow = (Object[]) this._rowTraverser.next();
			return true;
		}
		return false;
	}
	public int numColumns() throws DAOException {
		return this._columns.size();
	}
	public int numRows() {
		return this._rows.size();
	}
	public boolean previous() throws DAOException {
		if (this._rowTraverser == null) {
			this._rowTraverser = this._rows.listIterator();
		}

		if (this._rowTraverser.hasPrevious()) {
			this._currRow = (Object[]) this._rowTraverser.previous();
			return true;
		}
		return false;
	}
}
