package com.extensiblejava.dao.mediator;

import java.util.*;
import com.extensiblejava.dao.*;

public final class DAOMediator {
	private TreeMap insertables = new TreeMap();
	private int insertableNumber = 0;

	public DAOMediator() {
	}
	public void addInsertable(Insertable insertable) {
		insertableNumber++;
		insertables.put(new Integer(insertableNumber), insertable);
	}

	public void insert() throws DAOException, TransactionException {
		if (insertables.isEmpty()) {
			throw new DAOException("There are no Insertables registered");
		}
		Transaction trans = null;
		try {
			trans = new Transaction();
			Iterator inserts = insertables.values().iterator();
			while (inserts.hasNext()) {
				Insertable insertable = (Insertable) inserts.next();
				DataBaseDAO dao = DataBaseDAO.getInstance();
				Integer key = dao.executeInsert(insertable, trans);
				insertable.notifyListeners(key);
			}
			trans.commit();
		} catch (TransactionException e) {
			trans.rollback();
			throw e;
		} finally {
			trans.release();
		}
	}

	public DataCursor select(Selectable selectable)
		throws DAOException, TransactionException {
		Transaction trans = null;
		try {
			trans = new Transaction();
			DataBaseDAO dao = DataBaseDAO.getInstance();
			DataCursor cursor =
				dao.executeSelect(selectable, trans);
			trans.commit();
			return cursor;
		} catch (DAOException e) {
			trans.rollback();
			throw e;
		} finally {
			trans.release();
		}
	}
}