package com.extensiblejava.dao.test;

import junit.framework.TestCase;
import java.util.*;
import com.extensiblejava.dao.*;
import com.extensiblejava.dao.mediator.*;

public class TestMediator extends TestCase {
	
	public TestMediator(String s){
		super(s);
	}
	public static void main(String[] args) {
		junit.textui.TestRunner.run(TestMediator.class);

	}
	
	public void testInsertName() {
		DAOMediator mediator = new DAOMediator();
		
		InsertableAdapter person = new InsertableAdapter() {
			public String getInsertSQL() {
				return "insert into person (first, last, dob, ssn) " +
				"values ('Jason', 'Johnson', '1970-01-01', '999999999')";	
			}
			public SortedMap getSortedInsertValues() {
				return new TreeMap();	
			}
		};
		
		class EmployeeInsert extends InsertableAdapter implements KeyListener {
			private Integer keyValue;
			public String getInsertSQL() {
				return "insert into employee (mgr_id, rate, start_dt, position_cd, type_cd, department_cd, name_id) " +
				"values ('4', '65000.00', '1996-01-01', '01', '01', '01', ?)";	
			}
			public SortedMap getSortedInsertValues() {
				TreeMap map = new TreeMap();
				map.put(new Integer(1), keyValue);
				return map;	
			}
			public void updateKey(Integer keyValue) {
				this.keyValue = keyValue;
				
			}
		};
		EmployeeInsert employee = new EmployeeInsert();
		person.registerListener(employee);
		
		try {
			class MyKeyListener implements KeyListener {
				private Integer keyValue;
				public void updateKey(Integer keyValue) {
					this.keyValue = keyValue;
				}	
				public Integer getKey() {return this.keyValue;}
			}
			
			MyKeyListener newKey = new MyKeyListener();
			person.registerListener(newKey);
				
			mediator.addInsertable(person);
			mediator.addInsertable(employee);
			mediator.insert();
			
			
			DataCursor dc = mediator.select(new SingleEmployeeSelectable(newKey.getKey()));
			while (dc.next()) {
				System.out.println("NAME: " + dc.getValue(9) + " " + dc.getValue(11));
				System.out.println("START DT: " + dc.getValue("start_dt"));	
			}
		} catch (Exception e) {
			fail(e.toString());	
		}
		
		
	}
}

